%
% This file creates the Revised Closed loop nonlinear model for the F/A-18 
% aircraft. 
% To form the nonlinear closed loop model some approximations have been
% made.  All the approximations have been explained below. 


%==========================================================================
% Add Multipoly toolbox

%==========================================================================
% Generate the Open Loop Model
F18CubicNL_OLoop; 


%==========================================================================
% Initialize state and input variables

x = [beta;p1;r;phi;alpha;q];                        % State
xdot = [betad1; pd1; rd1; phid1; alphad1; qd1];     % State Derivative
u = [d_AIL; d_RUD; d_STAB];                         % Input



%==========================================================================
%  Form Linearized Plant
%  xdot = Flin*x + Glin*u
%  y    = Hlin*x + Jlin*u


G = jacobian(xdot,u);
F = xdot - G*u;

% Output Equation
Ay = qbar*S*(C_Y)/m/g;
y = [Ay; p1; r; alpha;beta;q;betad1];
J = jacobian(y,u);
H = y-J*u;


% Form Linearized Open Loop Plant
Flin = double(jacobian(cleanpoly(F,[],1),x));
Glin = double(cleanpoly(G,[],0));
Hlin = double(jacobian(cleanpoly(H,[],1),x));
Jlin = double(cleanpoly(J,[],0));
Pss = ss(Flin,Glin,Hlin,Jlin);


%==========================================================================
%  Form Revised Controller
%
%  xc = [xc1];
%  u = [d_AIL; d_RUD; d_STAB];
%  y = [ay; p; r; alpha;beta;q;betadot];
%
%  xc_dot = Ac*xc+Bc*y
%  u      = Cc*xc+Dc*u


pvar xc1 ;
xc = [xc1];                % Controller State


kbetadot = 2;              % beta dot feedback
kbeta    = 0.5;            % beta feedback

% Revised Controller Realization
Ac = [-1 ];
Bc = -[0 0 -4.9 0 0 0 0]; 
Cc =  [0 ;-1; 0];
Dc = -[0 -0.8 0 0 -kbeta 0 -kbetadot; 0.5 0 1.1 0 0 0 0;  0 0 0 0.8 0 2.5 0];
Css = ss(Ac,Bc,Cc,Dc);

%==========================================================================
% Linearized Closed Loop Plant
% Negative Feedback of Plin and C
CL = feedback(Pss,Css);
% damp(CL)



%==========================================================================
% Nonlinear Closed Loop Plant
%
% Negative Feedback of Pnonlin and C
% Define M:=I-J*Dc. inv(M) introduces a rational term (non-poly) into the
% closed loop model. This is approximated with a linear Taylor series.


M = (eye(7)+ J*Dc);

m11 = M(1,1);
m12 = M(1,2);
m13 = M(1,3);
m15 = M(1,5);

c2=cleanpoly(m11,[],0);
c1=diff(cleanpoly(m11,[],1),alpha);
m11inv = 1/c2 - c1/c2/c2*alpha;

Minv = polynomial(eye(7));
Minv(1,1) =  m11inv;
Minv(1,2) = -m12*m11inv;
Minv(1,3) = -m13*m11inv;
Minv(1,5) = -m15*m11inv;


%==========================================================================
% Form Nonlinear Closed Loop Revised Model

xcldot = [F-G*Cc*xc-G*Dc*Minv*(H-J*Cc*xc); ...
          Ac*xc+Bc*Minv*(H-J*Cc*xc)];

% Closed loop States
xcl = [x;xc];             


%==========================================================================
% For Nonlinear analysis we need a 3rd order polynomial model.
% However, the current model is more than a 3rd order.  So, we will perform
% some Least sqaure approximation to construct a 3rd order model. 


% Extract the terms that are more than 1st order.
A = double(jacobian(cleanpoly(xcldot,[],1),xcl));
e = xcldot - A*xcl;
e = cleanpoly(e,10e-8);    % ignore terms that are less than 10^-8. 

%==========================================================================
% Perform Least Square Approximation. 
% 
% Alpha-beta has been gridded in a range. Standard least square
% approximation has been performed within the alpha-beta gridded range for
% higher order terms. Look inside the below file for more details on how
% the approximation has been performed. 

LS_xcldot_Revised;


%==========================================================================
% Cubic Order Approximated Model.


clpdot(1) = [alpha^3  beta^3  alpha*beta^2  alpha^2*beta  alpha*beta ]*HabLSbd + ...
             [alpha^2  alpha 1]*HapLSbd*p1 + [alpha^2  alpha 1]*HarLSbd*r + ...
             [alpha^2  alpha 1]*HaxcLSbd*xc1 + ....
             A(1,:)*xcl +jacobian(e(1),q)*q +jacobian(e(1),phi)*phi;

clpdot(2) = [alpha^3  beta^3  alpha*beta^2  alpha^2*beta  alpha*beta ]*HabLSpd + ...
             [alpha^2  alpha 1]*HapLSpd*p1 + [alpha^2  alpha 1]*HarLSpd*r + ...
             [alpha^2  alpha 1]*HaxcLSpd*xc1 + ...
             A(2,:)*xcl +jacobian(e(2),phi)*phi +jacobian(e(2),q)*q;

clpdot(3) = [alpha^3  beta^3  alpha*beta^2  alpha^2*beta  alpha*beta ]*HabLSrd + ...
             [alpha^2  alpha 1]*HapLSrd*p1 + [alpha^2  alpha 1]*HarLSrd*r + ...
             [alpha^2  alpha 1]*HaxcLSrd*xc1 + ...
             A(3,:)*xcl +jacobian(e(3),phi)*phi + jacobian(e(2),q)*q;

xcldot = [clpdot';xcldot(4:7,:)];


% Clean up terms with coefficient less than 10^-10 

xcldot = cleanpoly(xcldot,10^-10); 


%==========================================================================
% Sanity Check on Eigenvalue Comparison

AA=double(jacobian(cleanpoly(xcldot,[],1),xcl));

fprintf('\n Sanity Check on Least Sqaure Approximation\n')
fprintf('\n Eigenvalue BEFORE Least Sqaure Approximation \n')
fprintf('================================================= \n')
eig(CL.a)

fprintf('\n Eigenvalue AFTER Least Sqaure Approximation \n')
fprintf('================================================= \n')

eig(AA)
 
 
 




