%  Search4UnstableTraj.m
%  This file attempts to solve via Monte Carlo search:
%      min  x0'*N*x0
%      subject to:  dx/dt = f(x) diverges from the initial condtion x0
%
%  The file randomly picks initial conditions on the ellipse x0'*N*x0 = g.
%  If a divergent trajectory is found then g is decreased by gfac and the
%  search continues.  A trajectory is considered "convergent" if it 
%  satisfies one of two conditions:
%     1) norm( x(t) ) < nsmall before Tfinal
%     2) abs( x_i(t) ) < xmall(i) for each state i before Tfinal 
%  Any trajectory not satisfying one of these two conditions is considered
%  "divergent".  The final solution should be checked to verify that the
%  trajectory actually does diverge from the initial condition.



%==========================================================================
% Add NL Tool Path
%==========================================================================


%==========================================================================
%                               Generate Model

% State Ordering 
% x     = [beta ;      p1      ; r     ;phi        ; alpha     ; q;   xc ];
% f     = [betad;      pd      ; rd    ;phid       ; alphad    ;qd;   xcd];
%==========================================================================

Baseline = 0;       % Set to zero for revised model 


if Baseline 
    load F18BaselineCubic;
else
    load F18RevisedCubic; 
end




%======================================================================
% Create Shape Function
d2r = pi/180;  r2d = 1 /d2r; 
Dmax = diag([5*d2r 20*d2r 5*d2r 45*d2r 25*d2r 25*d2r 25*d2r]);  
N = inv(Dmax^2);   %  Scale by inverse of max state values
N = N/max(N(:));   %  Normalize by the largest entry



%======================================================================
% Initialization of Simulation Parameter


Ntry = 5000000;         % Number of Monte carlo simulation runs 
Tfinal = 150;           % Simulation stopping time

gtry = 0.01;            % Size of the Initial Ellipsoid
gfac = 0.995;           % Decreasing Factor for the Ellipsoid

ns = length(x); 
x0min = zeros(ns,1);  

ind =1;                 % Index Initialization for Saving Data

%======================================================================
%                          Run Monte Carlo Simulation 
%======================================================================

for i1=1:Ntry;
    
    % Find random initial condition and scale so that x0'*N*x0 = gtry
    x0 = randn(ns,1);
    x0(end) = 0;   % Special for the A/C example--set controller state to zero
    scl = sqrt( (x0'*N*x0)/gtry );
    x0 = x0/scl;
    
    % Simulate and check for convergence
    [xtraj,xconv]= psim(f,x,x0,Tfinal);
    div = (xconv==0);
    
    if div
        
        % Found divergent trajectory
        fprintf('Divergent trajectory found at simulation step=%d \t gtry = %4.8f\n',i1,gtry)
        x0min = x0;
        
        % Save the Data once found divergent trajectories
        xun(:,ind) = x0min;
        betaun(ind) = gtry;
        %save FileName xun betaun i1 
        ind = ind +1; 
        
        % Decrease the ellipsoid shape since divergent trajectories found 
        gmax = gtry;
        gtry = gfac*gtry;
    end
    
        % Update Display every 10,000 iterations
        if rem(i1,ceil(Ntry/100))== 0
        fprintf('Current Iteration =%d \t gtry = %4.8f\n',i1,gtry)
        end
     
end





%-----------------------------------------------------------------------
% Simulate smallest IC and verify divergence
%-----------------------------------------------------------------------


[xtraj,xconv] = psim(f,x0min,Tfinal);
xtraj =xtraj{1};
plot(xtraj(1,:),xtraj(2:end,:))
legend(char(xcl));
[xcl x0min*r2d]
subs( xcl'*N*xcl, char(xcl), num2cell(x0min))
