%------------------------------------------------------------------
% Hahn's example with VS Iteration Initialized with Vlin
%   Compute an estimate of the region of attraction for Hahn's
%   example using the V-S iteration.  The iteration is initialized
%   with the Lyapunov function from linearized analysis.
%
% References: 
%  1) W. Hahn. Theory and application of Liapunov's direct method.
%     Prentice Hall, 1963.
%
%  2) W. Tan.  Nonlinear Control Analysis and Synthesis using
%     Sum-of-Squares Programming. Ph.D. Thesis, Univ. of California,
%     Berkeley, Spring 2006.
%     (See Section 3.1.4.2 for results on Hahn's example)
%------------------------------------------------------------------
clear all; close all;

% Create vector field
pvar x1 x2;
x = [x1;x2];
x1dot = -x1+2*x1^2*x2;
x2dot = -x2;
f = [x1dot; x2dot];

% Plot exact stability region
x1true = linspace(0.1,6);
figure(1); hold off;
plot(x1true,1./x1true,'b',-x1true,-1./x1true,'b');
axis([-6 6 -6 6]);
xlabel('x1');
ylabel('x2');
hold on;


% Create shape function and vector of Lyapunov function monomials
% The shape function is from W. Tan's Thesis (Sec 3.1.4.2)
Vdeg = 6;
p = x'*[14.47 18.55; 18.55 26.53]*x;
zV = monomials(x,2:Vdeg);    
z1maxd = ceil((Vdeg-p.maxdeg)/2);
z1 = monomials(x, 0:z1maxd ); 
z2 = monomials(x, 1:2 );
L2 = 1e-6*(x'*x);

% Construct Lyap function from linearization 
Vlin=linstab(f,x);

% Run V-s iteration
Nsteps = 50; 
ph = [];
opts = [];
opts.gmin = 0; 
opts.gmax = 50;
opts.L2 = L2;
for i1=1:Nsteps;    
    if i1==1
        V = Vlin;        
    else
        % roavstep
        [V,c] = roavstep(f,p,x,zV,b,g,s1,s2,opts);
        
        % Scale V by previous gamma.  This roughly normalizes V
        V = V/g;
    end
    
    % get gamma
    [gbnds,s2]=pcontain(jacobian(V,x)*f+L2,V,z2,opts);
    g = gbnds(1);

    % get beta
    [bbnds,s1]=pcontain(V-g,p,z1,opts);
    b = bbnds(1);
    
    fprintf('i1 = %d  \t beta = %4.3f\n',i1,b);

    % Plot V/p contours for ROA estimate 
    if  ~isempty(ph)
        delete(ph)
    end
    domain = [-6 6 -6 6];
    [C,ph(1)]=pcontour(V,g,domain,'r');
    [C,ph(2)]=pcontour(p,b,domain,'b');
    title(['Iteration = ' int2str(i1) ' beta = ' num2str(b)]);
    
    drawnow;    
end
