function [gbnds,sout]=pcontain(p0,p1,z,opts)
% function [gbnds,s]=pcontain(p0,p1,z,opts)
%
% DESCRIPTION 
%   This function maximizes g subject to the set containment constraint:
%           { x : p1(x)<= g } is a subset of { x : p0(x)<= 0 }
%   The set containment constraint is relaxed using a generalization of
%   the S-procedure, i.e. the set containment holds if there exists a 
%   function s(x) such that s and -( p0+(g-p1)*s ) are sums of squares.
%   This solved as the following SOS problem:
%        max g
%        subject to: s in SOS,  -( p0+(g-p1)*s ) in SOS
%   In general, bisection is required to find the optimal g since the
%   constraint is bilinear in g and s(x). The bisection stops when the 
%   upper/lower bounds on g are within a specified absolute tolerance.
%
% INPUTS 
%   p0,p1: Polynomials used to describe the sets in the containment.
%   z: Nz-by-1 column vector of monomials used to specify the 
%      polynomial decision variable s(x) in the Gram matrix form, 
%      s(x) = z(x)'*C*z(x). (Default: z=monomials(x,[mind,maxd])
%      where maxd is chosen such that maxdeg(p1*s) >= maxdeg(p0) 
%      and mind is chosen such that s.mindeg>=p0.mindeg )
%
%   opts: Options structure with the fields:
%     -gmin: Minimum value of g used in bisection (Default = -1e-3);
%     -gmax: Maximum value of g used in bisection (Default = 1e-3);
%     -bistol:  Bisection stop tolerance (Default: bistol = 1e-3)
%     -displayon: displayon=1 to display bisection iteration information.
%         Display information generated by sosopt and the SDP solver
%         is not affected by this option (Default: displayon=0).
%     -The remaining fields of opts are passed directly to sosopt.
%         Exceptions: opts.checkfeas = 'off' and opts.scaling = 'on'
%         are not allowed and will be over-ridden with
%         opts.checkfeas = 'fast' and opts.scaling = 'off'.
%     
% OUTPUTS
%   gbnds: 1-by-2 vector [glb, gub] providing a lower bound glb and
%      and upper bound gub on the maximum value of g. gbnds will be
%      empty if no feasible lower bound is found.
%   s: Multplier function proving set containment with glb.  s satisfies 
%      the constraints s  and  -(p0+(glb-p1)*s ) are SOS. s will be
%      empty if no feasible lower bound is found.
%
% SYNTAX
%   [gbnds,s]=pcontain(p0,p1)
%   [gbnds,s]=pcontain(p0,p1,z)
%   [gbnds,s]=pcontain(p0,p1,z,opts)
%
% EXAMPLE
%   % Maximize size of an ellipsoid contained in unit disk
%   pvar x1 x2;
%   x = [x1;x2];
%   p0 = x'*x-1;
%   p1 = 4*x1^2+9*x2^2;
%   z = 1;
%   [gbnds,sout]=pcontain(p0,p1,z)


%  PJS 4/27/2009  Initial coding based on getgamma and getbeta

% XXX-- (?) Add alternative formulation for an outer set approx:
%   min beta s.t. {p1<=0} contained in {p0<=beta}

% XXX-- Add exact solution for the case where p0, p1 are quadratic
% SOS forms?  This can be solved by computing a max gen. eigenval.

%------------------------------------------------------------------
% Error Checking
%------------------------------------------------------------------


%------------------------------------------------------------------
% Get options or set defaults for unspecified options
%------------------------------------------------------------------
if nargin==2
    z = [];
    opts = [];
elseif nargin==3
    opts = [];
end
if isempty(opts)
    opts.gmin = [];
    opts.gmax = [];
    opts.bistol = [];
    opts.displayon = [];
end
if ~isfield(opts,'gmin') || isempty(opts.gmin)
    gmin = -1e3;
else
    gmin = opts.gmin;
end
if ~isfield(opts,'gmax') || isempty(opts.gmax)
    gmax = 1e3;
else
    gmax = opts.gmax;
end
if ~isfield(opts,'bistol') || isempty(opts.bistol)
    bistol = 1e-3;
else
    bistol = opts.bistol;
end
if ~isfield(opts,'displayon') || isempty(opts.displayon)
    displayon = 0;
else    
    displayon = opts.displayon;
end
if ~isfield(opts,'checkfeas') || isempty(opts.checkfeas)
    opts.checkfeas = 'fast';
end
if strcmp(opts.checkfeas,'off')
    warning(['Bisection is based on a feasibility check. ' ...
        'opts.checkfeas = ''off'' is not allowed. ' ...
        'Setting opts.checkfeas to ''fast''.']);
    opts.checkfeas = 'fast';
end
if ~isfield(opts,'scaling') || isempty(opts.scaling)
    opts.scaling = 'off';
end
if strcmp(opts.scaling,'on')
    warning(['Setting opts.scaling to ''off''.']);
    opts.scaling = 'off';
end

%------------------------------------------------------------------
% Initialize variables
%------------------------------------------------------------------
glb = -inf;
gub = inf;
gbnds = [glb gub];
sout = polynomial([]);
dopt = [];

% Get polynomial variables
x = unique([p0.varname; p1.varname]);

% Set default multiplier
if isempty(z)
    % This is roughly based on guideilnes from Appendix A.1.1 of Ph.D. 
    % Thesis by Weehong Tan
    zmind = ceil(p0.mindeg/2);
    zmaxd = ceil( (p0.maxdeg-p1.maxdeg) / 2 );    
    if zmaxd<zmind
        z=1;
    else
        z = monomials(x, zmind:zmaxd );
    end
end

% Create decision variable s(x) and a dummy decision variable for g*s(x)
[svar,C] = polydecvar('c',z,'mat');
lz = size(C,1);
idx = find(tril(ones(lz)));            
C = char(C(idx));
[gsvar,GC] = polydecvar('gc',z,'mat');
GC = char(GC(idx));

% Initialize the two SOS constraints
%     s in SOS 
%     -( p0+(g-p1)*s ) = -(p0+g*s-p1*s) in SOS
% We'll use the info structure for a fast setup for each value of gamma.
sosconstr{1} = svar;
sosconstr{2} = -(p0+gsvar-p1*svar);
if isfield(opts,'solver');
    solver = opts.solver;
else
    solver = 'sedumi';
end
opts.solver = 'setup';
[info,dopt]=sosopt(sosconstr,x,opts);
opts.solver = solver;
info.opts.solver = solver;

if displayon
    fprintf('\n----------- Beginning Set Containment Iteration\n');
end        


%------------------------------------------------------------------
% Intialize lower bound from gmin
% Undocumented: If gmin is a vector then the code below will try to
% check all the values listed in gmin until it finds a feasible
% solution.  This can be used if one has a good guess for glb and 
% wants to speed up the bisection search with this initial guess.
%------------------------------------------------------------------
Ngmin = length(gmin);
if Ngmin > 1
    gmin = sort(gmin,'descend');
    for i1 = 1:Ngmin
        gtry = gmin(i1);
        [feas,d] = LOCALpcontain(info,gtry,C,GC);       
        if displayon
            fprintf('Verify Lower Bound: GTry = %4.3f\t',gtry)
            fprintf('Feas = %d\n',feas);
        end        
        if feas==0
            gub = gtry;
        else
            glb = gtry;
            dopt = d;
            break;
        end
    end    
    if feas==0
        % Set containment was invalid even for smallest val of gmin
        sout = polynomial([]);
        gbnds = [];
        return;
    end
else
    glb = gmin;
end

%------------------------------------------------------------------
% Initialize upper bound from gmax
% Undocumented: If gmax is a vector then the code below will try to
% check all the values listed in gmax until it finds an infeasible
% solution.  This can be used if one has a good guess for gub and 
% wants to speed up the bisection search with this initial guess.
%------------------------------------------------------------------
gmax = min(gmax,gub);
gmax = unique(gmax);
Ngmax = length(gmax);
if Ngmax > 1
    gmax = sort(gmax,'ascend');
    for i1 = 1:Ngmax
        gtry = gmax(i1);
        [feas,d] = LOCALpcontain(info,gtry,C,GC);
        if displayon
            fprintf('Verify Upper Bound: GTry = %4.3f\t',gtry)
            fprintf('Feas = %d\n',feas);
        end        
        if feas==1
            glb = gtry;
            dopt = d;
        else
            gub = gtry;
            break;
        end
    end    
    if feas==1
        % Set containment was valid even for largest val of gmax
        dopt = d;
        sout = subs(svar,dopt);
        gbnds = [gtry gtry];
        return;
    end
else
    gub = gmax;
end

%------------------------------------------------------------------
% Perform Bisection
%------------------------------------------------------------------
go = 1;  % Ensure at least one pass through
while (gub-glb>bistol) || (go==1)
    go = 0;
    
    % Set gamma level for feasibility problem
    gtry = (gub+glb)/2;
        
    % Solve feasibility problem
    [feas,d,sdpdata2] = LOCALpcontain(info,gtry,C,GC);
        
%     sosconstr{1} = svar;
%     sosconstr{2} = -(p0+(gtry-p1)*svar);
%     [info2,d] = sosopt(sosconstr,x,opts);
%     feas2 = info2.feas;
%     sdpdata1=info2.sdpdata;
%     [feas, feas2]
%     keyboard
    
    % Display Bisection Status
    if displayon 
        fprintf('GLB  = %4.5f \t',glb);
        fprintf('GTry = %4.5f \t',gtry);
        fprintf('GUB = %4.5f \t',gub);
        fprintf('Feas = %d\n',feas);
    end                        
    
    % Update bisection bounds
    if feas
        glb = gtry;
        dopt = d;
    else
        gub = gtry;
    end    
end

% Set output variable with upper/lower bounds
gbnds = [glb gub];
if isempty(dopt)
    gbnds = [];
    sout = [];
else
    sout = subs(svar,dopt);
end

return;

%-----------------------------------------------------------
% Local Function: 
%   Feasibility problem: Given g, find s(x) such that s 
%   in SOS and -( f+(g-p)*s ) in SOS.
%-----------------------------------------------------------
function [feas,dopt,sdpdata2] = LOCALpcontain(info,gtry,C,GC);

opts = info.opts;
decvars = info.sosdata.decvars;
dv2x  = info.sos2sdp.dv2x;

sdpdata = info.sdpdata;
A = sdpdata.A;    
b = sdpdata.b;
c = sdpdata.c;
K = sdpdata.K;

if strcmpi(opts.form,'image');
    % Modify SDP data for current value of gtry
    % GC are first Ndv dec vars and are treated as free vars in setup    
    % C are next Ndv dec vars and are mapped into PSD matrix var
    Ndv = length(C);
    [tf1,idx1]=ismember(C,decvars);
    [tf2,idx2]=ismember(GC,decvars);
%    idx1 = Ndv+1 : 2*Ndv;  % indices for C into decvars
%    idx2 = 1:Ndv;          % indices for GC into decvars
    dv2x2 = dv2x(idx1)-Ndv; % Mapping of C into sed vars of new problem

    Nsos = K.s(1)^2;
    szA = size(A);
    A2 = sparse(szA(1),Nsos);
    A2(:,dv2x2) = A(:,dv2x(idx1))+gtry*A(:,dv2x(idx2));

    Nslack = K.s(2)^2;    
    idx3 = szA(2)-Nslack+1:szA(2);    
    A2 = [A2 A(:,idx3)];
    sdpdata2.A = A2;    
    sdpdata2.b = b;    
    sdpdata2.c = sparse(size(A2,2),1);
    
    K2 = K;
    K2.f = 0;
    sdpdata2.K = K2;
    
    % Solve SDP
    [x,y,solverinfo] = solvesossdp(sdpdata2,opts); 
    sdpsol.x = x;
    sdpsol.y = y;
    sdpsol.solverinfo = solverinfo;
    info.sdpsol = sdpsol;
       
    % Feasibility Check
    if strcmp(opts.checkfeas,'fast') 
        feas = checkfeas(info);
    else
        Copt = [decvars(idx1) num2cell(x(dv2x2))];
        s1 = subs(info.sosdata.sosconstr{1}{1},Copt);
        w1 = info.sos2sdp.w{1};
        Q1 = reshape(x(1:Nsos),K2.s(1),K2.s(1));
        sossol = {s1,w1,Q1};
        
        GCopt = [decvars(idx2) num2cell(gtry*x(dv2x2))];
        s2 = subs(info.sosdata.sosconstr{2},[Copt;GCopt]);
        w2 = info.sos2sdp.w{2};
        Q2 = reshape(x(Nsos+1:end),K2.s(2),K2.s(2));
        sossol = [sossol; {s2 w2 Q2}];

        if strcmp(opts.checkfeas,'full')
            feas = checkfeas(sossol);
        elseif strcmpi(opts.checkfeas,'both');    
            feas = checkfeas(info,sossol);            
        else
            error('checkfeas option must be: full, both, or fast');
        end
    end

    % Create decision var cell array
    if feas ==1    
        dopt = [decvars(idx1) num2cell(x(dv2x2))];
    else
        dopt = [];
    end        
    
elseif strcmpi(opts.form,'kernel');
    % Modify SDP data for current value of gtry
    Ndv = length(C);
    [tf1,idx1]=ismember(C,decvars);
    [tf2,idx2]=ismember(GC,decvars);

    idx3= (2*Ndv+1):size(A,1);
    A2 = [A(idx1,:)+gtry*A(idx2,:); A(idx3,:)];
    sdpdata2.A = A2;
    
    Nnull = length(idx3);    
    sdpdata2.b = sparse(Ndv+Nnull,1);
    sdpdata2.c = c;
    sdpdata2.K = K;
    
    % Solve SDP
    [x,y,solverinfo] = solvesossdp(sdpdata2,opts); 
    sdpsol.x = x;
    sdpsol.y = y;
    sdpsol.solverinfo = solverinfo;
    info.sdpsol = sdpsol;
       
    % Feasibility Check
    if strcmp(opts.checkfeas,'fast') 
        feas = checkfeas(info);
    else
        % XXX Can't do 'full' feas check unless we form sossol
        Copt = [decvars(idx1) num2cell(y(1:Ndv))];
        s1 = subs(info.sosdata.sosconstr{1}{1},Copt);
        w1 = info.sos2sdp.w{1};
        idx = 1:K.s(1)^2;
        Q1 = c(idx) - A2(:,idx)'*y;        
        Q1 = reshape( Q1 , K.s(1), K.s(1) ); 
        sossol = {s1,w1,Q1};
        
        GCopt = [decvars(idx2) num2cell(gtry*y(1:Ndv))];
        s2 = subs(info.sosdata.sosconstr{2},[Copt;GCopt]);
        w2 = info.sos2sdp.w{2};
        idx = K.s(1)^2+(1:K.s(2)^2);
        Q2 = c(idx) - A2(:,idx)'*y;        
        Q2 = reshape( Q2 , K.s(2), K.s(2) ); 
        sossol = [sossol; {s2 w2 Q2}];

        if strcmp(opts.checkfeas,'full')
            feas = checkfeas(sossol);
        elseif strcmpi(opts.checkfeas,'both');    
            feas = checkfeas(info,sossol);            
        else
            error('checkfeas option must be: full, both, or fast');
        end        
    end
        
    % Create decision var cell array
    if feas ==1    
        dopt = [decvars(idx1) num2cell(y(1:Ndv))];
    else
        dopt = [];
    end    
end
   

