function [V,c] = roavstep(f,p,x,z,beta,gamma,s1,s2,opts)
% function [V,c] = roavstep(f,p,x,z,beta,gamma,s1,s2,opts)
%
% DESCRIPTION 
%   This function solves the V step of the ROA iteration.  Specifically,
%   the ROA estimation problem can be formulated as:
%      max beta
%      subject to:
%            V-L1  in SOS
%           -( (V-gamma)+(beta-p)*s1 ) in SOS 
%           -( (grad(V)*f+L2)+(gamma-V)*s2 ) in SOS
%
%   This function solves for a Lyapunov function V which satisfies the
%   three SOS constraints while holding all other variables fixed.
%
% INPUTS 
%   f: Vector field of the system dynamics (N-by-1 polynomial vector)
%   p: Shape function (scalar polynomial)
%   x: State vector (N-by-1 vector of pvars)
%   z: Nz-by-1 column vector of monomials used to specify the Lyapunov
%         function decision variable V(x) in the vector form, 
%         V(x) = c'*z(x). 
%   beta: Level set of shape function (scalar double)
%   gamma: Level set of Lyapunov function (scalar double)
%   s1,s2: Set containment multipliers (scalar polynomials)
%   opts: Options structure with the fields:
%     -L1: Function used to enforce strict positive definiteness of
%          V (Default = 1e-6*x'*x);
%     -L2: Function used to enforce strict negative definiteness of
%          dV/dt (Default = 1e-6*x'*x);
%     -solveropts:  SOS optimization options passed on to sosopt. 
%         (Default: solveropts = [])
%     
% OUTPUTS
%   V: Lyapunov function satisfying the three SOS constraints.  V will
%     be returned as empty if no feasible solution is found.
%   c: Nz-by-1 vector of coefficients such that V(x) = c'*z(x)
%
% SYNTAX
%   [V,c] = roavstep(f,p,x,z,beta,gamma,s1,s2,opts)

% PJS 4/28/2009   Initial coding based on getNewVfeas


%------------------------------------------------------------------
% Error Checking
%------------------------------------------------------------------


%------------------------------------------------------------------
% Get options or set defaults for unspecified options
%------------------------------------------------------------------
if nargin==8
    opts = [];
end
if isempty(opts)
    opts.L1 = [];
    opts.L2 = [];
    opts.solveropts = [];
end
if ~isfield(opts,'L1') || isempty(opts.L1)
    L1 = 1e-6*(x'*x);
else
    L1 = opts.L1;
end
if ~isfield(opts,'L2') || isempty(opts.L2)
    L2 = 1e-6*(x'*x);
else
    L2 = opts.L2;
end
if ~isfield(opts,'solveropts') || isempty(opts.solveropts)
    solveropts = [];
else    
    solveropts = opts.solveropts;
end

%------------------------------------------------------------------
% Set up and solve SOS feasibility problem
%------------------------------------------------------------------

% XXX If all monomials in L1 exist in V (project L1 onto z using
% poly2basis and then check for zero error) then we can define
% V = V0+L1 with V0 as a polydecvar in mat form.  Then we
% can constrain V0 to be SOS and use V in the other constraints
% This will simplify the SOS constraint on V.
% [tmp,e]=poly2basis(L1,z);
% 
% if 0 %(e==0)
%     % XXX This isn't working because we have the monomials in the
%     % vector form of V and we are trying to declare V using the 
%     % 'mat' form of polydecvar.
%         
%     % Create Lyapunov decision variable
%     [V0,c] = polydecvar('c',z.degmat/2,'mat');
%     V = V0+L1;
%     
%     % Constraint 1: V0 in SOS
%     sosconstr{1} = V0;    
% else    
%
%     % Create Lyapunov decision variable
%     [V,c] = polydecvar('c',z,'vec');
% 
%     % Constraint 1: V-L1  in SOS
%     sosconstr{1} = V - L1;
%    
% end

% Create Lyapunov decision variable
[V,c] = polydecvar('c',z,'vec');

% Constraint 1: V-L1  in SOS
sosconstr{1} = V - L1;

% Constraint 2: -( (V-gamma)+(beta-p)*s1 ) in SOS 
sosconstr{2} =  -( (V-gamma)+(beta-p)*s1 );

% Constraint 3: -( (grad(V)*f+L2)+(gamma-V)*s2 ) in SOS
Vdot = jacobian(V,x)*f;
sosconstr{3} =  -( (Vdot+L2)+(gamma-V)*s2 );

% Solve feasibility problem
[info,dopt,sossol] = sosopt(sosconstr,x,solveropts);        

% Create output
if info.feas
    V = subs(V,dopt(:,1),dopt(:,2));
    c = subs(c,dopt(:,1),dopt(:,2));
else
    V=[];
    c=[];
end

