function [roaconstr, opt, sys] = GetRoaOpts(fWithDel, x, zV, p, Bis)
%
% DESCRIPTION 
%   This function initializes the options structure for roa_est. 
%   [List key options for the user to modify]   
%
% INPUTS 
%   fWithDel: 
%   x: 
%   zV:   [optional] [default = quadratic]
%   p:    [optional] [default = x'*x]
%   Bis:  [optional]
%         Bis.flag == 1 - use SOS conditions that require bisection
%               Bis.s2deg - HALF of the degree of s2
%         Bis.flag ~= 1 - use SOS conditions that do not require bisection
%               Bis.r1deg - degree of r1
%               Bis.r2deg - degree of r2
%         
% OUTPUTS 
%   roaconstr:
%   opt: 
%   sys: 
%
% SYNTAX 
%   [roaconstr, opt, sys] = GetRoaOpts(fList, XList, zV, p, Bis)
%
% REFERENCE
%   Topcu, Packard, and Seiler,  "Local Stability Analysis Using 
%   simulations and Sum-of-Squares Programming," to appear in Automatica.
%

%------------------------------------------------------------------
% Initialize variables
%------------------------------------------------------------------

sys = fUnc2fAff( fWithDel, x); 

XList = x;
fList = sys.f;

if nargin<3 | isempty(zV)
    zV = monomials(XList,2);
end
if nargin<4 | isempty(p)
    p = XList'*XList;
end
if nargin<5
    opt.coordoptim.flag = 0;
else
    if isfield(Bis, 'flag')
        opt.coordoptim.flag = Bis.flag;
        ell = [isfield(Bis, 'r1deg') isfield(Bis, 'r2deg') isfield(Bis, 's2deg')];
        if (ell(3) & (ell(1) | ell(2))) | (Bis.flag & (ell(1) | ell(2))) | (~Bis.flag & (ell(3)))
            error('coordinate-wise opt type and multiplier choices do not match');
        end
        if Bis.flag
            if isfield(Bis, 's2deg')
                opt.coordoptim.s2deg = Bis.s2deg;
            end
        else
            if isfield(Bis, 'r1deg')
                opt.coordoptim.r1deg = Bis.r1deg;
            end
            if isfield(Bis, 'r2deg')
                opt.coordoptim.r2deg = Bis.r2deg;
            end
        end
    else
        ell = [isfield(Bis, 'r1deg') isfield(Bis, 'r2deg') isfield(Bis, 's2deg')];
        if ell(3) & (ell(1) | ell(2))
            error('coordinate-wise opt type and multiplier choices do not match');
        else
            if ell(3) 
                opt.coordoptim.flag = 1;
                opt.coordoptim.s2deg = Bis.s2deg;
            end
            if ell(2) 
                opt.coordoptim.flag = 0;
                opt.coordoptim.r2deg = Bis.r2deg;
            end
            if ell(1)
                opt.coordoptim.flag = 0;
                opt.coordoptim.r1deg = Bis.r1deg;
            end
        end
    
    end
end

        

%------------------------------------------------------------------
% Display Options
%------------------------------------------------------------------
opt.display.sedumi = 0;      % = 0 to turn off all sedumi display
opt.display.roaest = 0;      % = 0 to turn off roaest printouts
opt.display.BB = 1;
%------------------------------------------------------------------
% Solver Options
%------------------------------------------------------------------

% These options are used below to set options for findVfeas, 
% getgamma and getbeta
usesosp = 0;          % = 1 to use sosp otherwise the code uses
                      % SOSTOOLs everywhere except findVfeas.
sedopts.eps =1e-9;                    % Stop tolerance
sedopts.fid = opt.display.sedumi;     % =1 to display Sedumi data
sedopts.solver = 'sedumi';
sedopts.form = 'image';

%------------------------------------------------------------------
% Constraint Setup
%    This options structure describes the constraints used
%    to enforce V p.d. and Vdot n.d. on {x: p(x)<=beta}
%------------------------------------------------------------------
roaconstr.zV = zV;
roaconstr.p  = p;
roaconstr.mu = 0;
roaconstr.L1 = 1e-6*sum(XList.^2);
roaconstr.L2 = 1e-6*sum(XList.^2);
roaconstr.L2LP = sum(XList.^2);
%roaconstr.LinearStabFactor = 1e-6;
roaconstr = LocalGetMultipBasis(fList, XList, roaconstr, opt);


%------------------------------------------------------------------
% Options to save data
%------------------------------------------------------------------
opt.SaveResults.filename = '';

%------------------------------------------------------------------
% Parallelization options
%-------------------------------------------------------------------
opt.parallel.useCluster = 0;
opt.parallel.machinesToUse = [1:9];

%------------------------------------------------------------------
% Coordinatewise Optimization Options (Used in roa_est)
%------------------------------------------------------------------

opt.coordoptim.MaxIters = 20;
opt.coordoptim.IterStopTol = 0.03;
opt.coordoptim.solver = 'sedumi';

%------------------------------------------------------------------
% Simulation Options  (Used in SimLFG)
%------------------------------------------------------------------

opt.sim.flag = 1;
opt.sim.BetaInit = 5.0;                        % XXX How to set? 
opt.sim.BetaShrink = 0.95;
opt.sim.BetaMax = 5.0;                         % XXX How to set?
opt.sim.BetaGrow = 2.0;
opt.sim.NumConvTraj = 100+(4)^length(XList);   % XXX How to set?
opt.sim.tfinal = 500;                          % XXX How to set?
opt.sim.tfinalBackForDiv = 10;
opt.sim.dispEveryNth = 100;


event_params.nbig = 100;
event_params.nsmall = 0.03;
event_params.xbig = 100;
event_params.xsmall = 3e-2;
event_params.funchandle = [];
opt.sim.event_params = event_params;           % XXX How to set?

%------------------------------------------------------------------
% Linear Program Options (used in SimLFG:traj2lp)
%------------------------------------------------------------------
opt.lp.useVdot = 1;
opt.lp.useVdecr = 0;
opt.lp.decimN = 10; 
opt.lp.timestosolvelp = 40;
opt.lp.MinBetaToTryLP = 1e-3;
opt.lp.sampleForPositive = 0;
opt.lp.timesSampleLMI = 1000;
opt.lp.radiusSampleLMI = 0.1;

%------------------------------------------------------------------
% findVfeas Options (used in SimLFG:findVfeas)
%------------------------------------------------------------------
sedopts.eps =1e-9;                    % Stop tolerance
sedopts.fid = opt.display.sedumi;     % =1 to display Sedumi data
opt.findVfeas.sedopts = sedopts;
opt.findVfeas.usesosp = usesosp;
opt.findVfeas.feasonly = 1;
opt.findVfeas.useYalmip = 1;

%------------------------------------------------------------------
% sampleForV options
%------------------------------------------------------------------
opt.sampleV.nofVs = 1;
opt.sampleV.nofptsEachTime = 5;
opt.sampleV.useLin = 1;
opt.sampleV.useVsos = 1;
opt.sampleV.Factor = 0.5;
opt.sampleV.runtoNmax = 0;


%------------------------------------------------------------------
% getgamma Options 
%------------------------------------------------------------------
opt.getgamma.bistol = 5e-3;
opt.getgamma.maxgamma = 10;            % XXX How to set? V~1 should be boundary
opt.getgamma.maxgammaBase = opt.getgamma.maxgamma;
opt.getgamma.mingamma = 0;
opt.getgamma.display = opt.display.roaest;
opt.getgamma.usesosp = usesosp;
opt.getgamma.sedopts = sedopts;
%------------------------------------------------------------------
% getbeta Options 
%------------------------------------------------------------------
opt.getbeta.bistol = 5e-2;
opt.getbeta.maxbeta = 20;                      % XXX How to set?
opt.getbeta.minbeta = 0;
opt.getbeta.display = opt.display.roaest;
opt.getbeta.usesosp = usesosp;
opt.getbeta.sedopts = sedopts;

%------------------------------------------------------------------
% options for robustness analysis
% I am just creating this field, I guess in later versions
% it will make sense to collect some other options under this field
%------------------------------------------------------------------

opt.cellBeta.Extrafs = 0;
opt.cellBeta.assessPriorV = 0;

%------------------------------------------------------------------
% Branch-and-bound Options 
%------------------------------------------------------------------

opt.BB.max_iter = 20;
%opt.BB.CenterVertexBetaGap = 0.1;
opt.BB.SubdivisionRule = 'SubdivisionRule';
opt.BB.FactorNomSimCellBeta = 0.5;
opt.BB.FactorNomStartPriorCellBeta = 0.75;
% % This next one will be replaced by opt.cellBeta.Extrafs in
% % the coming versions.
% opt.BB.NumberExtraNominal = 0;
opt.BB.runBB = 0;
opt.BB.logToDiv = zeros(length(sys.delvector));



%-------------------------------------------------------------------
% Local functions
%-------------------------------------------------------------------

function roaconstr = LocalGetMultipBasis(fList, XList, roaconstr, opt);
zV = roaconstr.zV;
p = roaconstr.p;
if opt.coordoptim.flag
    
    z2mindeg = 1;
    if isfield(opt.coordoptim, 's2deg')
        z2maxdeg = max([ceil( (fList.maxdeg-1)/2 ),opt.coordoptim.s2deg]);
    else 
        z2maxdeg = ceil( (fList.maxdeg-1)/2 );
    end
    roaconstr.z2 = monomials(XList, z2mindeg:z2maxdeg );

    % Let s1(x) = z1(x)'*Q1*z1(x) and choose deg[z1] so that deg[p*s1]>=deg[V]
    z1mindeg = 0;
    z1maxdeg = max([0;ceil( (zV.maxdeg-p.maxdeg) / 2 )]);
    roaconstr.z1 = monomials(XList, z1mindeg:z1maxdeg );

    % Choose deg[z3] = 0 where s3(x) = z3(x)'*Q3*z3(x).
    roaconstr.z3= polynomial(1);
else
    zr1mindeg = 0;
    zr2mindeg = 0;
    if isfield(opt.coordoptim, 'r1deg')
         zr1maxdeg = opt.coordoptim.r1deg;
    else
        zr1maxdeg = 2;
    end
    if isfield(opt.coordoptim, 'r2deg')
         zr2maxdeg = opt.coordoptim.r2deg;
    else
        zr2maxdeg = 2;
    end
    roaconstr.zr1 = monomials(XList, zr1mindeg:zr1maxdeg );
    roaconstr.zr2 = monomials(XList, zr2mindeg:zr2maxdeg );
end

%------------------------------------------------------------------


