
function outputs = wrapper(sys,c,roaconstr,opt)

% function outputs = wrapper(sys,c,roaconstr,opt);
%
% DESCRIPTION
%  This function is a wrapper for the ROA routines and
%  depending on the structure of the inputs it does one
%  of the following:
%   (i)   Given (non-uncertain) vector field compute the ROA
%               [If sys.delvector is empty]
%   (ii)  Given uncertain vector field and a point in Delta,
%             compute the ROA for the system corresponding to that point
%               [If size(c,2)==length(sys.delvector). In this case,
%                 i-th entry of c will be substituted for the i-th 
%                 entry of delvector.]
%   (iii) Given uncertain vector field and Delta,
%             compute the robust ROA
%               [If size(c,2)==2*length(sys.delvector) and 
%                   opt.BB.runBB == 0. In this case, 
%                   c = [d1min d1max d2min d2max ... dmmin dmmax] ] 
%   (iv)  Given uncertain vector field and Delta,
%             run the branch-and-bound refinement.
%               [If size(c,2)==2*length(sys.delvector) and 
%                   opt.BB.runBB == 1. In this case, 
%                   c = [d1min d1max d2min d2max ... dmmin dmmax] ] 
%
% INPUTS
%  sys: struct with fields
%        [ not all of these fields are used in every case ]
%        [ GetRoaOpts.m creates this struct ]
%        delvector  : vector of uncertain parameters (polys)
%        fWithDel   : nx1 poly array, vector field with uncertain
%                     parameters
%        x          : state vector (poly)
%        nonaffine  : non-affine functions of uncertain parameters (polys)
%        f          : nx(m+mpu+1) matrix of poly, fWithDel decomposed so that
%                     fWithDel = f(:,1) + sum_{i=1}^{m}{f(:,i+1)*delvector(i)} +
%                     sum_{j=1}^{mpu}{f(:,j+m+1)*nonaffine(j)}
%  c  : double array either empty or 1xm or 1x2m (see above which one is
%       used in which case.
%  roaconstr : struct with parameters used in forming V, multipliers, etc.
%       [ GetRoaOpts.m creates this struct ]
%  opt : struct with the options for ROA estimation.
%       [ GetRoaOpts.m creates this struct ]
%
% OUTPUTS
%  struct containing the inputs to the wrapper and the outputs 
%  from the ROA estimation routine.



if isfield(opt,'Vinitial')
    opt.sim.flag = 0;
end

 
outputs.inputs.sys = sys;
outputs.inputs.ini_cell = c;
outputs.inputs.roaconstr = roaconstr;
outputs.inputs.opt = opt;

if isempty(sys.delvector)
    if size(sys.f,2)>1
        [BetaBnds, V, info] = RoaEstNew(sys.f, sys.x, roaconstr, opt, sys.f);
    else
        [BetaBnds, V, info] = RoaEstNew(sys.f, sys.x, roaconstr, opt);
    end
    RoaEstInfo.BetaBnds = BetaBnds;
    RoaEstInfo.V = V;
    RoaEstInfo.info = info;
    outputs.RoaEstInfo = RoaEstInfo;
elseif size(c,2)==length(sys.delvector)
    fin = subsLocal(sys.fWithDel,sys.delvector,c);
    [BetaBnds, V, info] = RoaEst(fin, sys.x, roaconstr, opt);
    RoaEstInfo.BetaBnds = BetaBnds;
    RoaEstInfo.V = V;
    RoaEstInfo.info = info;
    outputs.RoaEstInfo = RoaEstInfo;
elseif size(c,2)==2*length(sys.delvector)
    if ~opt.BB.runBB 
        [fNOM{1},fVER{1}] = getf(sys,c);
        if opt.cellBeta.Extrafs > 0
            if opt.cellBeta.Extrafs >= size(fVER{1},2)-1
                opt.cellBeta.Extrafs = -1;
            else
                NoOfVert = size(fVER{1},2);
                vvv = ceil(NoOfVert*rand(opt.cellBeta.Extrafs,1));
                vvv = unique(vvv);
                fNOM{1} = [fNOM{1} fVER{1}(:,vvv)];
            end
        end
        if opt.cellBeta.Extrafs == -1
            fNOM{1} = fVER{1};
        end
        [BetaOut,VOut,GammaOut, UpBOut, NominalBetaOut,infoOut] =...
            cellBetaCenter(fNOM,fVER, sys.x, roaconstr, opt);
        CellBetaInfo.BetaOut = BetaOut;
        CellBetaInfo.VOut = VOut;
        CellBetaInfo.GammaOut = GammaOut;
        CellBetaInfo.UpBOut = UpBOut;
        CellBetaInfo.NominalBetaOut = NominalBetaOut;
        CellBetaInfo.infoOut = infoOut;
        outputs.CellBetaInfo = CellBetaInfo;
        outputs.fNOM = fNOM;
        outputs.fVER = fVER;
    else
        [Beta_vec, Active, Mcell, Gamma_vec, Vvec, UpB_vec, NominalBetaVec,info] =...
            BandBmanager(sys, sys.x, c, roaconstr, opt);
        BBInfo.Beta_vec = Beta_vec;
        BBInfo.Active = Active;
        BBInfo.Mcell = Mcell;
        BBInfo.Gamma_vec = Gamma_vec;
        BBInfo.Vvec = Vvec;
        BBInfo.UpB_vec = UpB_vec;
        BBInfo.NominalBetavec = NominalBetaVec;
        BBInfo.info = info;
        outputs.BBInfo = BBInfo;
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fin = subsLocal(f,dvec,c)

fin = f;
for i1 = 1:length(dvec)
    fin = subs(fin,{dvec(i1)},{c(i1)});
end

        
            

    

